#include <math.h>

#include "CVect3D.hpp"

using namespace std;

const CVect3D CVect3D::zero( 0.0, 0.0, 0.0 );

void CVect3D::set( double _x, double _y, double _z )
 {
    x = _x;
    y = _y;
    z = _z;
    return;
 }

void CVect3D::reset()
 {
    x = 0.0;
    y = 0.0;
    z = 0.0;
    return;
 }

double CVect3D::len() const
 {
    return sqrt( this->lensqr() );
 }

double CVect3D::lensqr() const
 {
    return (x*x + y*y + z*z);
 }

CVect3D &CVect3D::neg()
 {
    x = -x;
    y = -y;
    z = -z;
    return *this;
 }

CVect3D &CVect3D::add( const CVect3D &p )
 {
    x += p.x;
    y += p.y;
    z += p.z;
    return *this;
 }

CVect3D &CVect3D::sub( const CVect3D &p )
 {
    x -= p.x;
    y -= p.y;
    z -= p.z;
    return *this;
 }

double CVect3D::muls( const CVect3D &p ) const
 {
    return (x*p.x + y*p.y + z*p.z);
 }

CVect3D &CVect3D::mulv( const CVect3D &p )
 {
    double _x;
    double _y;
    _x = y*p.z - z*p.y;
    _y = z*p.x - x*p.z;
    z = x*p.y - y*p.x;
    y = _y;
    x = _x;
    return *this;
 }

CVect3D &CVect3D::mul( double k )
 {
    x *= k;
    y *= k;
    z *= k;
    return *this;
 }

CVect3D &CVect3D::norm()
 {
    double r = len();
    
    x /= r;
    y /= r;
    z /= r;
    return *this;
 }

CVect3D &CVect3D::rotate ( float angle, const CVect3D &rotvec )
{
    CVect3D temp;

    // Calculate the sine and cosine of the angle once
    float cosTheta = (float)cos(angle);
    float sinTheta = (float)sin(angle);

    // Find the new x position for the new rotated point
    temp.x  = (cosTheta + (1 - cosTheta) * rotvec.x * rotvec.x)           * x;
    temp.x += ((1 - cosTheta) * rotvec.x * rotvec.y - rotvec.z * sinTheta)* y;
    temp.x += ((1 - cosTheta) * rotvec.x * rotvec.z + rotvec.y * sinTheta)* z;

    // Find the new y position for the new rotated point
    temp.y  = ((1 - cosTheta) * rotvec.x * rotvec.y + rotvec.z * sinTheta)* x;
    temp.y += (cosTheta + (1 - cosTheta) * rotvec.y * rotvec.y)           * y;
    temp.y += ((1 - cosTheta) * rotvec.y * rotvec.z - rotvec.x * sinTheta)* z;

    // Find the new z position for the new rotated point
    temp.z  = ((1 - cosTheta) * rotvec.x * rotvec.z - rotvec.y * sinTheta)* x;
    temp.z += ((1 - cosTheta) * rotvec.y * rotvec.z + rotvec.x * sinTheta)* y;
    temp.z += (cosTheta + (1 - cosTheta) * rotvec.z * rotvec.z)           * z;

    *this = temp;
    return *this;
}

CVect3D  CVect3D::operator-() const {return CVect3D( -x, -y, -z );}
CVect3D &CVect3D::operator+=( const CVect3D &p ) {return add( p );}
CVect3D &CVect3D::operator-=( const CVect3D &p ) {return sub( p );}
CVect3D &CVect3D::operator^=( const CVect3D &p ) {return mulv( p );}
CVect3D &CVect3D::operator*=( double k ) {return mul( k );}
CVect3D  CVect3D::operator~() const
 {
    CVect3D o( *this );
    o.norm();
    return o;
 }

CVect3D operator+( const CVect3D &a, const CVect3D &b )
 {
    CVect3D o( a );
    o.add( b );
    return o;
 }

CVect3D operator-( const CVect3D &a, const CVect3D &b )
 {
    CVect3D o( a );
    o.sub( b );
    return o;
 }

double operator*( const CVect3D &a, const CVect3D &b )
 {
    return a.muls( b );
 }

CVect3D operator^( const CVect3D &a, const CVect3D &b )
 {
    CVect3D o( a );
    o.mulv( b );
    return o;
 }
 
CVect3D operator*( const CVect3D &a, double k )
 {
    CVect3D o( a );
    o.mul( k );
    return o;
 }

bool operator==( const CVect3D &a, const CVect3D &b ) {return(a.x == b.x && a.y == b.y && a.z == b.z);}
bool operator!=( const CVect3D &a, const CVect3D &b ) {return(a.x != b.x || a.y != b.y && a.z == b.z);}

void CVect3D::swap( CVect3D *a, CVect3D *b )
 {
    double tmp;
    
    tmp = a->x; a->x = b->x; b->x = tmp;
    tmp = a->y; a->y = b->y; b->y = tmp;
    tmp = a->z; a->z = b->z; b->z = tmp;
    return;
 }
