#ifndef __CVect3D_hpp__
#define __CVect3D_hpp__

#ifndef NULL
#define NULL 0x00
#endif


class CVect3D
 {
  public:
    double x;
    double y;
    double z;
    
    CVect3D( double _x = 0.0, double _y = 0.0, double _z = 0.0 ) {x = _x; y = _y; z = _z;}
    ~CVect3D() {}
    
    void set( double, double, double );
    void reset();
    bool isZero() const {return (x == 0.0 && y == 0.0 && z == 0.0);}
    double len() const;
    double lensqr() const;                                                                          // len^2

    CVect3D &neg();
    CVect3D &add( const CVect3D & );
    CVect3D &sub( const CVect3D & );
    double muls( const CVect3D & ) const;                                                           // scalar multiplication
    CVect3D &mulv( const CVect3D & );                                                               // vector multiplication
    CVect3D &mul( double k );                                                                       // length multiplication
    CVect3D &norm();
    CVect3D &rotate( float angle, const CVect3D &rotvec );                                          // rotation around rotvec

    CVect3D operator-() const;                                                                      // inverted vector
    CVect3D &operator+=( const CVect3D & );
    CVect3D &operator-=( const CVect3D & );
    CVect3D &operator^=( const CVect3D & );                                                         // vector multiplication
    CVect3D &operator*=( double k );                                                                // length multiplication
    CVect3D operator~() const;                                                                      // normalized vector
    
    friend CVect3D operator+( const CVect3D &, const CVect3D & );
    friend CVect3D operator-( const CVect3D &, const CVect3D & );
    friend double  operator*( const CVect3D &, const CVect3D & );
    friend CVect3D operator^( const CVect3D &, const CVect3D & );
    friend CVect3D operator*( const CVect3D &, double k );
    friend bool operator==( const CVect3D &, const CVect3D & );
    friend bool operator!=( const CVect3D &, const CVect3D & );

    static void swap( CVect3D *, CVect3D * );
    
    static const CVect3D zero;
 };

#endif
